unit Floppy32;

interface
Uses Windows;


// Define the media types supported by the driver.


Type T_MEDIA_TYPE = (Unknown,                // Format is unknown
                    F5_1Pt2_512,            // 5.25", 1.2MB,  512 bytes/sector
                    F3_1Pt44_512,           // 3.5",  1.44MB, 512 bytes/sector
                    F3_2Pt88_512,           // 3.5",  2.88MB, 512 bytes/sector
                    F3_20Pt8_512,           // 3.5",  20.8MB, 512 bytes/sector
                    F3_720_512,             // 3.5",  720KB,  512 bytes/sector
                    F5_360_512,             // 5.25", 360KB,  512 bytes/sector
                    F5_320_512,             // 5.25", 320KB,  512 bytes/sector
                    F5_320_1024,            // 5.25", 320KB,  1024 bytes/sector
                    F5_180_512,             // 5.25", 180KB,  512 bytes/sector
                    F5_160_512,             // 5.25", 160KB,  512 bytes/sector
                    RemovableMedia,         // Removable media other than floppy
                    FixedMedia,             // Fixed hard disk media
                    F3_120M_512             // 3.5", 120M Floppy
                   );


// The following structure is returned on an IOCTL_DISK_GET_DRIVE_GEOMETRY
// request and an array of them is returned on an IOCTL_DISK_GET_MEDIA_TYPES
// request.

type T_DISK_GEOMETRY = record
                         Cylinders:LARGE_INTEGER;
                         MediaType:T_MEDIA_TYPE;
                         TracksPerCylinder:DWORD;
                         SectorsPerTrack:DWORD;
                         BytesPerSector:DWORD ;
                       end;




type T_FORMAT_PARAMETERS = record
                             MediaType:DWORD;
                             StartCylinderNumber:DWORD;
                             EndCylinderNumber:DWORD;
                             StartHeadNumber:DWORD;
                             EndHeadNumber:DWORD;
                           end;


Type TPhysicalFloppy = Class
                       Private
                       Protected
                         FDiskOpen:Boolean;
                         FDrive:Byte;
                         FDrive_Handle:DWORD;
                         FDiskGeometry:T_DISK_GEOMETRY;
                         Procedure CheckVersion;Virtual;Abstract;
                       Public
                         Constructor Create;
                         Destructor Destroy;override;
                         Function ReadFloppySector(Var Data;Drive,Side,
                                  Track,Sector:byte):DWORD;Virtual;Abstract;
                         Function ReadFloppyTrack(Var Data;Drive,Side,
                                  Track,SectorCount:byte):DWORD;Virtual;Abstract;
                         Function ReadFloppy(Var Data;Drive:Byte):DWORD;Virtual;Abstract;
                         Function WriteFloppySector(Var Data;Drive,Side,
                                  Track,Sector:byte):DWORD;Virtual;Abstract;
                         Function WriteFloppyTrack(Var Data;Drive,Side,
                                  Track,SectorCount:byte):DWORD;Virtual;Abstract;
                         Function WriteFloppy(Var Data;Drive:Byte):DWORD;Virtual;Abstract;
                         Function ReadNextFloppyTrack(Var Data):DWORD;Virtual;Abstract;
                         Function ReadNextFloppySector(Var Data):DWORD;Virtual;Abstract;
                         Function WriteNextFloppyTrack(Var Data):DWORD;Virtual;Abstract;
                         Function WriteNextFloppySector(Var Data):DWORD;Virtual;Abstract;
                         Function DiskSize(Drive:Byte):DWORD;Virtual;Abstract;
                         Function NewDisk(Drive:Byte):DWORD;Virtual;Abstract;
                         Function GetDiskGeometry(Var Geometry:T_DISK_GEOMETRY;Drive:Byte):DWORD;Virtual;Abstract;
                         Function SeekLogicalSector(Drive:Byte;Sector:Dword):DWORD;Virtual;Abstract;
                         Function LockDrive(Drive:Byte):DWORD;Virtual;Abstract;
                         Function UnLockDrive(Drive:Byte):DWORD;Virtual;Abstract;
                         Function FormatTracks(Drive:Byte;var FormatParameters:T_FORMAT_PARAMETERS):DWORD;Virtual;Abstract;
                         Procedure SenseMediaType;Virtual;Abstract;
                         Property Geometry:T_DISK_GEOMETRY read FDiskGeometry;
                       End;

implementation

{ TPhysicalFloppy }


Function CTL_CODE(iDeviceType,iFunction,iMethod,iAccess:DWORD):DWORD;
Begin
  Result:=((iDeviceType shl 16) or (iAccess shl 14) or (iFunction shl 2) or (iMethod));
end;




constructor TPhysicalFloppy.Create;
begin
  Inherited;
  CheckVersion;
  FDiskOpen:=False;
  FDrive:=255;
  FDrive_Handle:=INVALID_HANDLE_VALUE;
end;

destructor TPhysicalFloppy.Destroy;
begin
  if FDiskOpen then CloseHandle(FDrive_Handle);
  inherited;
end;

end.

(*
//
// Define the input buffer structure for the driver, when
// it is called with IOCTL_DISK_FORMAT_TRACKS.
//

typedef struct _FORMAT_PARAMETERS {
   MEDIA_TYPE MediaType;
   DWORD StartCylinderNumber;
   DWORD EndCylinderNumber;
   DWORD StartHeadNumber;
   DWORD EndHeadNumber;
} FORMAT_PARAMETERS, *PFORMAT_PARAMETERS;

//
// Define the BAD_TRACK_NUMBER type. An array of elements of this type is
// returned by the driver on IOCTL_DISK_FORMAT_TRACKS requests, to indicate
// what tracks were bad during formatting. The length of that array is
// reported in the `Information' field of the I/O Status Block.
//

typedef WORD   BAD_TRACK_NUMBER;
typedef WORD   *PBAD_TRACK_NUMBER;

//
// Define the input buffer structure for the driver, when
// it is called with IOCTL_DISK_FORMAT_TRACKS_EX.
//

typedef struct _FORMAT_EX_PARAMETERS {
   MEDIA_TYPE MediaType;
   DWORD StartCylinderNumber;
   DWORD EndCylinderNumber;
   DWORD StartHeadNumber;
   DWORD EndHeadNumber;
   WORD   FormatGapLength;
   WORD   SectorsPerTrack;
   WORD   SectorNumber[1];
} FORMAT_EX_PARAMETERS, *PFORMAT_EX_PARAMETERS;
*)
 //
// The following structure is returned on an IOCTL_DISK_GET_PARTITION_INFO
// and an IOCTL_DISK_GET_DRIVE_LAYOUT request.  It is also used in a request
// to change the drive layout, IOCTL_DISK_SET_DRIVE_LAYOUT.
//

typedef struct _PARTITION_INFORMATION {
    LARGE_INTEGER StartingOffset;
    LARGE_INTEGER PartitionLength;
    DWORD HiddenSectors;
    DWORD PartitionNumber;
    BYTE  PartitionType;
    BOOLEAN BootIndicator;
    BOOLEAN RecognizedPartition;
    BOOLEAN RewritePartition;
} PARTITION_INFORMATION, *PPARTITION_INFORMATION;

//
// IoControlCode values for disk devices.
//

#define IOCTL_DISK_BASE                 FILE_DEVICE_DISK
#define IOCTL_DISK_GET_DRIVE_GEOMETRY   CTL_CODE(IOCTL_DISK_BASE, 0x0000, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_GET_PARTITION_INFO   CTL_CODE(IOCTL_DISK_BASE, 0x0001, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_SET_PARTITION_INFO   CTL_CODE(IOCTL_DISK_BASE, 0x0002, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_DISK_GET_DRIVE_LAYOUT     CTL_CODE(IOCTL_DISK_BASE, 0x0003, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_SET_DRIVE_LAYOUT     CTL_CODE(IOCTL_DISK_BASE, 0x0004, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_DISK_VERIFY               CTL_CODE(IOCTL_DISK_BASE, 0x0005, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_FORMAT_TRACKS        CTL_CODE(IOCTL_DISK_BASE, 0x0006, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_DISK_REASSIGN_BLOCKS      CTL_CODE(IOCTL_DISK_BASE, 0x0007, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_DISK_PERFORMANCE          CTL_CODE(IOCTL_DISK_BASE, 0x0008, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_IS_WRITABLE          CTL_CODE(IOCTL_DISK_BASE, 0x0009, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_LOGGING              CTL_CODE(IOCTL_DISK_BASE, 0x000a, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_FORMAT_TRACKS_EX     CTL_CODE(IOCTL_DISK_BASE, 0x000b, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_DISK_HISTOGRAM_STRUCTURE  CTL_CODE(IOCTL_DISK_BASE, 0x000c, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_HISTOGRAM_DATA       CTL_CODE(IOCTL_DISK_BASE, 0x000d, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_HISTOGRAM_RESET      CTL_CODE(IOCTL_DISK_BASE, 0x000e, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_REQUEST_STRUCTURE    CTL_CODE(IOCTL_DISK_BASE, 0x000f, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_DISK_REQUEST_DATA         CTL_CODE(IOCTL_DISK_BASE, 0x0010, METHOD_BUFFERED, FILE_ANY_ACCESS)

#if(_WIN32_WINNT >= 0x0400)
#define IOCTL_DISK_CONTROLLER_NUMBER    CTL_CODE(IOCTL_DISK_BASE, 0x0011, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// IOCTL support for SMART drive fault prediction.
//

#define SMART_GET_VERSION               CTL_CODE(IOCTL_DISK_BASE, 0x0020, METHOD_BUFFERED, FILE_READ_ACCESS)
#define SMART_SEND_DRIVE_COMMAND        CTL_CODE(IOCTL_DISK_BASE, 0x0021, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define SMART_RCV_DRIVE_DATA            CTL_CODE(IOCTL_DISK_BASE, 0x0022, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#endif /* _WIN32_WINNT >= 0x0400 */


//
// The following device control codes are common for all class drivers.  The
// functions codes defined here must match all of the other class drivers.
//
// Warning: these codes will be replaced in the future by equivalent
// IOCTL_STORAGE codes
//

#define IOCTL_DISK_CHECK_VERIFY     CTL_CODE(IOCTL_DISK_BASE, 0x0200, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_MEDIA_REMOVAL    CTL_CODE(IOCTL_DISK_BASE, 0x0201, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_EJECT_MEDIA      CTL_CODE(IOCTL_DISK_BASE, 0x0202, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_LOAD_MEDIA       CTL_CODE(IOCTL_DISK_BASE, 0x0203, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_RESERVE          CTL_CODE(IOCTL_DISK_BASE, 0x0204, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_RELEASE          CTL_CODE(IOCTL_DISK_BASE, 0x0205, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_FIND_NEW_DEVICES CTL_CODE(IOCTL_DISK_BASE, 0x0206, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_GET_MEDIA_TYPES CTL_CODE(IOCTL_DISK_BASE, 0x0300, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
